/*
This file was generated by the following command:

cfunctions ed-trans-char.c

*/
#ifndef CFH_ED_TRANS_CHAR_H
#define CFH_ED_TRANS_CHAR_H

#line 12 "ed-trans.c.tmpl"
int distance_char_trans (text_fuzzy_t* tf);

#endif /* CFH_ED_TRANS_CHAR_H */
