/*
This file was generated by the following command:

cfunctions edit-distance-int.c

*/
#ifndef CFH_EDIT_DISTANCE_INT_H
#define CFH_EDIT_DISTANCE_INT_H

#line 12 "edit-distance.c.tmpl"
int distance_int (text_fuzzy_t* tf);

#endif /* CFH_EDIT_DISTANCE_INT_H */
